package cd.casic.module.terminal.host.config.model;

import cd.casic.module.terminal.common.GenericsDataModel;
import cd.casic.module.terminal.common.UpdatePasswordAction;
import jakarta.validation.constraints.*;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 *
 * 主机 SSH 配置
 *
 * @author Yuru Pu
 * @version 1.0
 * @since 2025/7/15 15:22
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class HostSshConfigModel implements GenericsDataModel, UpdatePasswordAction {

    /**
     * 主机端口
     */
    @NotNull
    @Min(value = 1)
    @Max(value = 65535)
    private Integer port;

    /**
     * 用户名
     */
    @Size(max = 128)
    private String username;

    /**
     * 认证方式
     */
    @NotBlank
    @Size(max = 12)
    private String authType;

    /**
     * 密码
     */
    private String password;

    /**
     * 身份id
     */
    private Long identityId;

    /**
     * 密钥id
     */
    private Long keyId;

    /**
     * 连接超时时间
     */
    @NotNull
    @Min(value = 1)
    @Max(value = 100000)
    private Integer connectTimeout;

    /**
     * SSH输出编码
     */
    @NotBlank
    @Size(max = 12)
    private String charset;

    /**
     * 文件名称编码
     */
    @NotBlank
    @Size(max = 12)
    private String fileNameCharset;

    /**
     * 文件内容编码
     */
    @NotBlank
    @Size(max = 12)
    private String fileContentCharset;

    /**
     * 是否使用新密码 仅参数
     */
    private Boolean useNewPassword;

    /**
     * 是否已设置密码 仅返回
     */
    private Boolean hasPassword;

}
