package cd.casic.module.terminal.utils;

import cd.casic.module.terminal.common.holder.SpringHolder;
import cd.casic.module.terminal.define.config.AppSftpConfig;
import cd.casic.module.terminal.host.transfer.SftpFileBackupParams;
import cn.orionsec.kit.lang.utils.Booleans;
import cn.orionsec.kit.lang.utils.Strings;
import cn.orionsec.kit.net.host.sftp.SftpExecutor;
import cn.orionsec.kit.net.host.sftp.SftpFile;
import com.alibaba.fastjson.JSON;



/**
 * sftp 工具类
 *
 * @author Jiahang Li
 * @version 1.0.0
 * @since 2024/5/8 16:17
 */
public class SftpUtils {

    private static final AppSftpConfig appSftpConfig = SpringHolder.getBean(AppSftpConfig.class);

    private SftpUtils() {
    }

    /**
     * 检查上传文件是否存在 并且执行响应策略
     *
     * @param executor executor
     * @param path     path
     */
    public static void checkUploadFilePresent(SftpExecutor executor, String path) {
        // 重复不备份
        if (!Booleans.isTrue(appSftpConfig.getUploadPresentBackup())) {
            return;
        }
        // 检查文件是否存在
        SftpFile file = executor.getFile(path);
        if (file != null) {
            // 文件存在则备份
            SftpFileBackupParams backupParams = new SftpFileBackupParams(file.getName());
            String target = Strings.format(appSftpConfig.getUploadBackupFileName(), JSON.parseObject(JSON.toJSONString(backupParams)));
            // 移动
            executor.move(path, target);
        }
    }

}
